@@ -1,5 +1,18 @@
-// Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
-// SPDX-License-Identifier: Apache-2.0
+/*********************************************************************************************************************
+ Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
+
+ Licensed under the Apache License, Version 2.0 (the "License").
+ You may not use this file except in compliance with the License.
+ You may obtain a copy of the License at
+
+ http://www.apache.org/licenses/LICENSE-2.0
+
+ Unless required by applicable law or agreed to in writing, software
+ distributed under the License is distributed on an "AS IS" BASIS,
+ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ See the License for the specific language governing permissions and
+ limitations under the License.
+ ******************************************************************************************************************** */
 
 import * as fs from "fs";
 import * as path from "path";
@@ -224,34 +237,23 @@ export class NxMonorepoProject extends TypeScriptProject {
   /**
    * @inheritDoc
    */
-  preSynthesize() {
-    super.preSynthesize();
-
-    // Add workspaces for each subproject
-    if (this.package.packageManager === NodePackageManager.PNPM) {
-      new YamlFile(this, "pnpm-workspace.yaml", {
-        readonly: true,
-        obj: {
-          packages: this.subProjects.map((subProject) =>
-            path.relative(this.outdir, subProject.outdir)
-          ),
-        },
-      });
-    } else {
-      this.package.addField("workspaces", {
-        packages: this.subProjects.map((subProject) =>
-          path.relative(this.outdir, subProject.outdir)
-        ),
-        nohoist: this.workspaceConfig?.noHoist,
-      });
-    }
+  synth() {
+    this.validateSubProjects();
+    this.updateWorkspace();
+    this.synthesizeNonNodePackageJson();
+    super.synth();
+  }
 
+  /**
+   * Ensures subprojects don't have a default task and that all packages use the same package manager.
+   */
+  private validateSubProjects() {
     this.subProjects.forEach((subProject: any) => {
       // Disable default task on subprojects as this isn't supported in a monorepo
       subProject.defaultTask?.reset();
 
       if (
-        (subProject instanceof NodeProject || subProject.package) &&
+        isNodeProject(subProject) &&
         subProject.package.packageManager !== this.package.packageManager
       ) {
         throw new Error(
@@ -262,27 +264,12 @@ export class NxMonorepoProject extends TypeScriptProject {
   }
 
   /**
-   * @inheritDoc
+   * For non-node projects, a package.json is required in order to be discovered by NX.
    */
-  synth() {
-    // Check to see if a new subProject was added
-    const newSubProject = this.subProjects.find(
-      (subProject) => !fs.existsSync(subProject.outdir)
-    );
-
-    // Need to synth before generating the package.json otherwise the subdirectory won't exist
-    newSubProject && super.synth();
-
+  private synthesizeNonNodePackageJson() {
     this.subProjects
-      .filter(
-        (subProject) =>
-          !subProject.tryFindObjectFile("package.json") ||
-          (fs.existsSync(`${subProject.outdir}/package.json`) &&
-            JSON.parse(
-              fs.readFileSync(`${subProject.outdir}/package.json`).toString()
-            ).__pdk__)
-      )
-      .forEach((subProject) => {
+      .filter((subProject: any) => !isNodeProject(subProject))
+      .forEach((subProject: Project) => {
         // generate a package.json if not found
         const manifest: any = {};
         manifest.name = subProject.name;
@@ -302,9 +289,41 @@ export class NxMonorepoProject extends TypeScriptProject {
           readonly: true,
         });
       });
-
-    super.synth();
   }
+
+  /**
+   * Add a submodule entry to the appropriate workspace file.
+   */
+  private updateWorkspace() {
+    // Add workspaces for each subproject
+    if (this.package.packageManager === NodePackageManager.PNPM) {
+      new YamlFile(this, "pnpm-workspace.yaml", {
+        readonly: true,
+        obj: {
+          packages: this.subProjects.map((subProject) =>
+            path.relative(this.outdir, subProject.outdir)
+          ),
+        },
+      });
+    } else {
+      this.package.addField("workspaces", {
+        packages: this.subProjects.map((subProject) =>
+          path.relative(this.outdir, subProject.outdir)
+        ),
+        nohoist: this.workspaceConfig?.noHoist,
+      });
+    }
+  }
+}
+
+/**
+ * Determines if the passed in project is of type NodeProject.
+ *
+ * @param project Project instance.
+ * @returns true if the project instance is of type NodeProject.
+ */
+function isNodeProject(project: any) {
+  return project instanceof NodeProject || project.package;
 }
 
 function getPluginPath() {
